from transformers import WhisperProcessor, WhisperForConditionalGeneration
import librosa
from jaseci.actions.live_actions import jaseci_action
import numpy as np
from fastapi import HTTPException
import traceback
import requests
import uuid
import os
import base64
from typing import List

SUPPORTED_LANGUAGES = [
    "en",
    "zh",
    "de",
    "es",
    "ru",
    "ko",
    "fr",
    "ja",
    "pt",
    "tr",
    "pl",
    "ca",
    "nl",
    "ar",
    "sv",
    "it",
    "id",
    "hi",
    "fi",
    "vi",
    "iw",
    "uk",
    "el",
    "ms",
    "cs",
    "ro",
    "da",
    "hu",
    "ta",
    "no",
    "th",
    "ur",
    "hr",
    "bg",
    "lt",
    "la",
    "mi",
    "ml",
    "cy",
    "sk",
    "te",
    "fa",
    "lv",
    "bn",
    "sr",
    "az",
    "sl",
    "kn",
    "et",
    "mk",
    "br",
    "eu",
    "is",
    "hy",
    "ne",
    "mn",
    "bs",
    "kk",
    "sq",
    "sw",
    "gl",
    "mr",
    "pa",
    "si",
    "km",
    "sn",
    "yo",
    "so",
    "af",
    "oc",
    "ka",
    "be",
    "tg",
    "sd",
    "gu",
    "am",
    "yi",
    "lo",
    "uz",
    "fo",
    "ht",
    "ps",
    "tk",
    "nn",
    "mt",
    "sa",
    "lb",
    "my",
    "bo",
    "tl",
    "mg",
    "as",
    "tt",
    "haw",
    "ln",
    "ha",
    "ba",
    "jw",
    "su",
]


def setup(size: str = "tiny"):
    global model, processor
    model = WhisperForConditionalGeneration.from_pretrained(f"openai/whisper-{size}")
    processor = WhisperProcessor.from_pretrained(f"openai/whisper-{size}")


setup()


def get_array(audio_file: str = None) -> np.ndarray:
    """Get numpy array from audio file"""
    audio, _ = librosa.load(audio_file, sr=16000)
    return audio


def download(url: str = None) -> str:
    """Download audio file from url"""
    r = requests.get(url, allow_redirects=True)
    filename = str(uuid.uuid4()) + ".wav"
    open(filename, "wb").write(r.content)
    return filename


@jaseci_action(act_group=["stt"], allow_remote=True)
def transcribe(
    language: str = "en",
    array: List[float] = None,
    audio_file: str = None,
    url: str = None,
) -> str:
    try:
        global model, processor

        if language not in SUPPORTED_LANGUAGES:
            raise ValueError(f"Language {language} not supported")

        if array is not None:
            audio_array = np.array(array)
        elif audio_file is not None:
            audio_array = get_array(audio_file)
        elif url is not None:
            downloaded_audio_file = download(url)
            audio_array = get_array(downloaded_audio_file)
            os.remove(downloaded_audio_file)
        else:
            raise ValueError("Must provide array, audio_file, or url")

        model.config.forced_decoder_ids = processor.get_decoder_prompt_ids(
            language=language, task="transcribe"
        )
        input_features = processor(
            audio_array, sampling_rate=16000, return_tensors="pt"
        ).input_features
        generated_ids = model.generate(input_features)
        transcription = processor.batch_decode(generated_ids, skip_special_tokens=True)[
            0
        ]
        return transcription.strip()
    except Exception as e:
        print(traceback.format_exc())
        raise HTTPException(status_code=500, detail=str(e))


@jaseci_action(act_group=["stt"], allow_remote=True)
def translate(
    language: str = "fr",
    array: List[float] = None,
    audio_file: str = None,
    url: str = None,
) -> str:
    try:
        global model, processor

        if language not in SUPPORTED_LANGUAGES:
            raise ValueError(f"Language {language} not supported")

        if array is not None:
            audio_array = np.array(array)
        elif audio_file is not None:
            audio_array = get_array(audio_file)
        elif url is not None:
            downloaded_audio_file = download(url)
            audio_array = get_array(downloaded_audio_file)
            os.remove(downloaded_audio_file)
        else:
            raise ValueError("Must provide array, audio_file, or url")

        forced_decoder_ids = processor.get_decoder_prompt_ids(
            language=language, task="translate"
        )
        input_features = processor(
            audio_array, sampling_rate=16000, return_tensors="pt"
        ).input_features
        generated_ids = model.generate(
            input_features, forced_decoder_ids=forced_decoder_ids
        )
        transcription = processor.batch_decode(generated_ids, skip_special_tokens=True)[
            0
        ]
        return transcription.strip()
    except Exception as e:
        print(traceback.format_exc())
        raise HTTPException(status_code=500, detail=str(e))


@jaseci_action(act_group=["stt"], allow_remote=True)
def audio_to_array(
    audio_file: str = None, url: str = None, base64_str: str = None
) -> List[float]:
    try:
        if audio_file is not None:
            audio_array = get_array(audio_file)
        elif url is not None:
            downloaded_audio_file = download(url)
            audio_array = get_array(downloaded_audio_file)
            os.remove(downloaded_audio_file)
        elif base64_str is not None:
            # save the base64 string to a file
            filename = str(uuid.uuid4()) + ".wav"
            with open(filename, "wb") as fh:
                fh.write(base64.b64decode(base64_str))
            audio_array = get_array(filename)
            os.remove(filename)
        else:
            raise ValueError("Must provide audio_file or url")
        return audio_array.tolist()
    except Exception as e:
        print(traceback.format_exc())
        raise HTTPException(status_code=500, detail=str(e))


if __name__ == "__main__":
    from jaseci.actions.remote_actions import launch_server

    launch_server(port=8000)



import os
import time
import subprocess
from pathlib import Path

from TTS.api import TTS
from TTS.utils.synthesizer import Synthesizer
from TTS.utils.manage import ModelManager


def get_models_file_path():
    return Path(__file__).parent / "models.json"


voice_cloning_model = "tts_models/multilingual/multi-dataset/your_tts"

manager = ModelManager(
    models_file=get_models_file_path(), progress_bar=True, verbose=False
)


def download_model_by_name(model_name: str):
    """
    Download the model from the checkpoint path.

    Parameters:
    -----------
    model_name: String, name of the model.

    Return:
    -----------
    model_path: String, Path to downloaded checkpoint file
    config_path: String, model configuration file path
    vocoder_path: String, vocoder model path
    vocoder_config_path: String vocoder configuration path
    """
    model_path, config_path, model_item = manager.download_model(model_name)
    if model_item["default_vocoder"] is None:
        return model_path, config_path, None, None
    vocoder_path, vocoder_config_path, _ = manager.download_model(
        model_item["default_vocoder"]
    )
    return model_path, config_path, vocoder_path, vocoder_config_path


def load_model_by_name(model_name: str, gpu: bool = False):
    """
    Load the downloaded model.

    Parameters:
    -----------
    model_name: String, name of the model.
    gpu: Boolean, if gpu is available.

    Return:
    -----------
    synthesizer: A class object of synthesizer.
    """
    model_path, config_path, vocoder_path, vocoder_config_path = download_model_by_name(
        model_name
    )

    # None values are fetch from the model
    synthesizer = Synthesizer(
        tts_checkpoint=model_path,
        tts_config_path=config_path,
        tts_speakers_file=None,
        tts_languages_file=None,
        vocoder_checkpoint=vocoder_path,
        vocoder_config=vocoder_config_path,
        encoder_checkpoint=None,
        encoder_config=None,
        use_cuda=gpu,
    )

    return synthesizer


# defining the synthesizer for voice cloning model.
synthesizer = load_model_by_name(voice_cloning_model)



from setuptools import setup, find_packages
from setuptools.command.install import install
import atexit
import sys
import platform
import os


MODULES = ["stt", "vc_tts"]
INSTALLABLE_MODULES = []


def _post_install():
    UNSUPPORTED_MODULES = [
        module for module in MODULES if module not in INSTALLABLE_MODULES
    ]
    if UNSUPPORTED_MODULES:
        print(
            "\033[93m"
            + f"{UNSUPPORTED_MODULES} modules are not supported on your system. "
            + "\033[0m"
        )


class new_install(install):
    def __init__(self, *args, **kwargs):
        super(new_install, self).__init__(*args, **kwargs)
        atexit.register(_post_install)


def get_ver():
    with open(os.path.join("./jac_speech", "VERSION")) as version_file:
        return version_file.read().strip()


def get_os():
    if sys.platform == "darwin" and platform.machine() == "arm64":
        return ".darwin"
    return ""


def get_extras_requires():
    extras_requires = {"all": []}
    platform = get_os()
    for module in MODULES:
        if not os.path.exists(
            os.path.join("jac_speech", module, f"requirements{platform}.txt")
        ):
            continue
        with open(
            os.path.join("jac_speech", module, f"requirements{platform}.txt")
        ) as req_file:
            extras_requires[module] = req_file.read().splitlines()
            extras_requires["all"].extend(extras_requires[module])
        INSTALLABLE_MODULES.append(module)
    return extras_requires


setup(
    name="jac_speech",
    version=get_ver(),
    packages=find_packages(include=["jac_speech", "jac_speech.*"]),
    install_requires=["jaseci", "pytest>=7.0.1,<7.1", "pytest-order>=1.0.1,<1.1"],
    extras_require=get_extras_requires(),
    package_data={
        "": ["*.json", "*.cfg", "VERSION", "*.yaml", "requirements.txt"],
    },
    author="Jason Mars",
    author_email="jason@jaseci.org",
    url="https://github.com/Jaseci-Labs/jaseci",
    cmdclass={"install": new_install},
)
